'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var backendCommon = require('@backstage/backend-common');
var express = require('express');
var Router = require('express-promise-router');
var requestLibrary = require('request');
var clientNode = require('@kubernetes/client-node');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var express__default = /*#__PURE__*/_interopDefaultLegacy(express);
var Router__default = /*#__PURE__*/_interopDefaultLegacy(Router);
var requestLibrary__default = /*#__PURE__*/_interopDefaultLegacy(requestLibrary);

var GitOpsDeliveryTool = /* @__PURE__ */ ((GitOpsDeliveryTool2) => {
  GitOpsDeliveryTool2["NONE"] = "none";
  GitOpsDeliveryTool2["CONFIG_SYNC"] = "config-sync";
  return GitOpsDeliveryTool2;
})(GitOpsDeliveryTool || {});
var ClusterLocatorMethodType = /* @__PURE__ */ ((ClusterLocatorMethodType2) => {
  ClusterLocatorMethodType2["CURRENT_CONTEXT"] = "current-context";
  ClusterLocatorMethodType2["IN_CLUSTER"] = "in-cluster";
  return ClusterLocatorMethodType2;
})(ClusterLocatorMethodType || {});
var ClusterLocatorAuthProvider = /* @__PURE__ */ ((ClusterLocatorAuthProvider2) => {
  ClusterLocatorAuthProvider2["CURRENT_CONTEXT"] = "current-context";
  ClusterLocatorAuthProvider2["GOOGLE"] = "google";
  ClusterLocatorAuthProvider2["OIDC"] = "oidc";
  ClusterLocatorAuthProvider2["SERVICE_ACCOUNT"] = "service-account";
  return ClusterLocatorAuthProvider2;
})(ClusterLocatorAuthProvider || {});
var OIDCTokenProvider = /* @__PURE__ */ ((OIDCTokenProvider2) => {
  OIDCTokenProvider2["NONE"] = "none";
  OIDCTokenProvider2["GOOGLE"] = "google";
  OIDCTokenProvider2["OKTA"] = "okta";
  return OIDCTokenProvider2;
})(OIDCTokenProvider || {});
const getResourcesNamespace = (config) => {
  const namespace = config.getString("resourcesNamespace");
  return namespace;
};
const getGitOpsDeliveryTool = (config) => {
  const gitOpsTool = config.getString("gitOpsDeliveryTool");
  if (!Object.values(GitOpsDeliveryTool)) {
    throw new Error(`Unknown gitOpsDeliveryTool, ${gitOpsTool}`);
  }
  return gitOpsTool;
};
const getClusterLocatorMethodType = (config) => {
  const methodType = config.getString("clusterLocatorMethod.type");
  if (!Object.values(ClusterLocatorAuthProvider)) {
    throw new Error(`Unknown clusterLocatorMethod.type, ${methodType}`);
  }
  return methodType;
};
const getClusterLocatorMethodAuthProvider = (config) => {
  const authProvider = config.getString("clusterLocatorMethod.authProvider");
  if (!Object.values(ClusterLocatorAuthProvider)) {
    throw new Error(
      `Unknown clusterLocatorMethod.authProvider, ${authProvider}`
    );
  }
  return authProvider;
};
const getClusterLocatorMethodOIDCTokenProvider = (config) => {
  const authProvider = getClusterLocatorMethodAuthProvider(config);
  if (authProvider === "oidc" /* OIDC */) {
    const oidcTokenProvider = config.getString(
      "clusterLocatorMethod.oidcTokenProvider"
    );
    if (!Object.values(OIDCTokenProvider)) {
      throw new Error(
        `Unknown clusterLocatorMethod.oidcTokenProvider, ${oidcTokenProvider}`
      );
    }
    return oidcTokenProvider;
  }
  return "none" /* NONE */;
};
const getClusterLocatorMethodServiceAccountToken = (config) => {
  const authProvider = getClusterLocatorMethodAuthProvider(config);
  if (authProvider === "service-account" /* SERVICE_ACCOUNT */) {
    return config.getString("clusterLocatorMethod.serviceAccountToken");
  }
  return "";
};

const getKubernetesConfig = (clusterLocatorMethodType) => {
  const kubeConfig = new clientNode.KubeConfig();
  switch (clusterLocatorMethodType) {
    case ClusterLocatorMethodType.IN_CLUSTER:
      kubeConfig.loadFromCluster();
      break;
    case ClusterLocatorMethodType.CURRENT_CONTEXT:
      kubeConfig.loadFromDefault();
      break;
    default:
      throw new Error(
        `Unknown cluster locator method type, ${clusterLocatorMethodType}`
      );
  }
  return kubeConfig;
};

const getClientAuthentication = (authProvider, oidcTokenProvider) => {
  switch (authProvider) {
    case ClusterLocatorAuthProvider.GOOGLE:
      return "google";
    case ClusterLocatorAuthProvider.OIDC:
      switch (oidcTokenProvider) {
        case OIDCTokenProvider.GOOGLE:
          return "oidc.google";
        case OIDCTokenProvider.OKTA:
          return "oidc.okta";
        default:
          throw new Error(
            `Client authenticaiton cannot be determined for OIDC token provider ${oidcTokenProvider}`
          );
      }
    case ClusterLocatorAuthProvider.SERVICE_ACCOUNT:
    case ClusterLocatorAuthProvider.CURRENT_CONTEXT:
      return "none";
    default:
      throw new Error(
        `Client authenticaiton cannot be determined for auth provider ${authProvider}`
      );
  }
};
async function createRouter({
  config,
  logger
}) {
  const router = Router__default["default"]();
  router.use(express__default["default"].json());
  const cadConfig = config.getConfig("configAsData");
  const namespace = getResourcesNamespace(cadConfig);
  const gitOpsTool = getGitOpsDeliveryTool(cadConfig);
  const clusterLocatorMethodType = getClusterLocatorMethodType(cadConfig);
  const clusterLocatorMethodAuthProvider = getClusterLocatorMethodAuthProvider(cadConfig);
  const oidcTokenProvider = getClusterLocatorMethodOIDCTokenProvider(cadConfig);
  const kubeConfig = getKubernetesConfig(clusterLocatorMethodType);
  const currentCluster = kubeConfig.getCurrentCluster();
  if (!currentCluster) {
    throw new Error(`Current cluster is not set`);
  }
  const serviceAccountToken = getClusterLocatorMethodServiceAccountToken(cadConfig);
  const clientAuthentication = getClientAuthentication(
    clusterLocatorMethodAuthProvider,
    oidcTokenProvider
  );
  logger.info(`Using '${clientAuthentication}' for client authentication`);
  logger.info(`Using '${namespace}' as the resources namespace`);
  const k8sApiServerUrl = currentCluster.server;
  const healthCheck = (_, response) => {
    response.send({ status: "ok" });
  };
  const getFeatures = (_, response) => {
    response.send({
      authentication: clientAuthentication,
      namespace,
      gitOps: gitOpsTool
    });
  };
  const getFunctionCatalog = (_, response) => {
    requestLibrary__default["default"](
      "https://catalog.kpt.dev/catalog-v2.json",
      (error, catalogResponse, catalogBody) => {
        if (error) {
          response.status(500);
        } else {
          response.status(catalogResponse.statusCode);
          response.send(catalogBody);
        }
      }
    );
  };
  const proxyKubernetesRequest = (request, response) => {
    var _a, _b;
    logger.info(`${request.method} ${request.url}`);
    const requestOptions = {
      baseUrl: k8sApiServerUrl,
      url: request.url,
      method: request.method,
      body: Object.keys(request.body).length === 0 ? void 0 : JSON.stringify(request.body)
    };
    kubeConfig.applyToRequest(requestOptions);
    const useEndUserAuthz = clientAuthentication !== "none";
    if (useEndUserAuthz) {
      requestOptions.headers = (_a = requestOptions.headers) != null ? _a : {};
      requestOptions.headers.authorization = request.headers.authorization;
    }
    const useServiceAccount = clusterLocatorMethodAuthProvider === ClusterLocatorAuthProvider.SERVICE_ACCOUNT;
    if (useServiceAccount) {
      requestOptions.headers = (_b = requestOptions.headers) != null ? _b : {};
      requestOptions.headers.authorization = `Bearer ${serviceAccountToken}`;
    }
    requestLibrary__default["default"](requestOptions, (k8Error, k8Response, k8Body) => {
      if (k8Error) {
        response.status(500);
      } else {
        response.status(k8Response.statusCode);
        response.send(k8Body);
      }
    });
  };
  router.get("/health", healthCheck);
  router.get("/v1/features", getFeatures);
  router.get("/v1/function-catalog", getFunctionCatalog);
  router.get("/*", proxyKubernetesRequest);
  router.post("/*", proxyKubernetesRequest);
  router.put("/*", proxyKubernetesRequest);
  router.patch("/*", proxyKubernetesRequest);
  router.delete("/*", proxyKubernetesRequest);
  router.use(backendCommon.errorHandler());
  return router;
}

exports.createRouter = createRouter;
//# sourceMappingURL=index.cjs.js.map
